package com.huawei.dli.flink.demo.sink;

import java.sql.*;

import org.apache.flink.configuration.Configuration;
import org.apache.flink.runtime.execution.SuppressRestartsException;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DWSSink extends RichSinkFunction<String> {
  private static final Logger logger = LoggerFactory.getLogger(DWSSink.class);

  private String dbUrl;
  private String userName;
  private String password;
  private String tableName;

  private transient Connection connection;
  private transient Statement statement;

  public DWSSink(String dbUrl, String userName, String password, String tableName) {
    this.dbUrl = dbUrl;
    this.userName = userName;
    this.password = password;
    this.tableName = tableName;
  }

  @Override
  public void open(Configuration parameters) {
    connectDb();
  }

  @Override
  public void invoke(String in) {
    try {
      String insertSql = String.format("insert into %s values('%s')", tableName, in);
      statement.execute(insertSql);
    } catch (Exception ex) {
      logger.error("Failed to write data to dcs", ex);
    }
  }

  @Override
  public void close() throws Exception {
    super.close();
    if (statement != null) {
      statement.close();
    }
    if (connection != null) {
      connection.close();
    }
  }

  private void connectDb() {
    try {
      // 加载JDBC驱动
      Class.forName("org.postgresql.Driver");

      // 获取数据库连接
      connection = DriverManager.getConnection(dbUrl, userName, password);
      statement = connection.createStatement();
      logger.info("Successfully connected to dws.");
    } catch (Exception ex) {
      logger.error("Failed to connect to db", ex);
      throw new SuppressRestartsException(ex);
    }
  }
}
