package com.huawei.dli.flink.demo.sink;

import java.security.PrivilegedExceptionAction;

import com.huawei.dli.flink.demo.utils.LoginUtil;
import org.apache.flink.runtime.state.FunctionInitializationContext;
import org.apache.flink.streaming.connectors.fs.bucketing.BasePathBucketer;
import org.apache.flink.streaming.connectors.fs.bucketing.BucketingSink;
import org.apache.hadoop.security.UserGroupInformation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HdfsSink extends BucketingSink<String> {
  private static final Logger logger = LoggerFactory.getLogger(HdfsSink.class);

  private String tmpDir = "/tmp/hdfsSink.";
  private String principalName;

  public HdfsSink(String basePath, String userPrincipal, long batchRolloverInterval) {
    super(basePath);
    this.principalName = userPrincipal;

    setBucketer(new BasePathBucketer<>());
    setBatchRolloverInterval(batchRolloverInterval);
  }

  @Override
  public void initializeState(FunctionInitializationContext context) throws Exception {
    // Do kerberos authentication
    org.apache.hadoop.conf.Configuration config = new org.apache.hadoop.conf.Configuration();
    config.set("hadoop.security.authentication", "kerberos");
    config.set("hadoop.rpc.protection", "privacy");
    String userKeytabFile = LoginUtil.createFile(tmpDir, "user.keytab", this.getClass());
    String krb5File = LoginUtil.createFile(tmpDir, "krb5.conf", this.getClass());
    LoginUtil.login(principalName, userKeytabFile, krb5File, config);

    setFSConfig(config);
    UserGroupInformation.getLoginUser().doAs(new PrivilegedExceptionAction<Void>() {
      public Void run() throws Exception {
        try {
          HdfsSink.super.initializeState(context);
        } catch (Exception e) {
          logger.error("Failed to initial filesystem.", e);
          throw e;
        }
        return null;
      }
    });
  }

  @Override
  public void close() throws Exception {
    super.close();
    try {
      LoginUtil.deleteFile(tmpDir, "user.keytab");
      LoginUtil.deleteFile(tmpDir, "krb5.conf");
    } catch (Exception e) {
      logger.error("Failed to close sink.", e);
    }
  }
}
