package com.huawei.dli.demo

import org.apache.spark.sql.{SaveMode, SparkSession}
import org.apache.spark.sql.types.{IntegerType, StringType, StructField, StructType}

object SparkRedisDemoDf {
  case class Person(id: Int, name: String, age: Int)

  def main(args: Array[String]): Unit = {
    val spark = SparkSession
      .builder()
      .appName("datasource_redis")
      .config("spark.redis.host", "192.168.0.201")
      .config("spark.redis.port", "6379")
      .config("spark.redis.auth", "xxxx")
      .getOrCreate()

    val sc = spark.sparkContext

    // DataFrame only support the redis hash type
    // *********** read the redis hash type data with DataFrame *******************

    // Prepare the data via redis cli:
    // hset customer:1 name John age 30
    // hset customer:2 name Peter age 45

    // read data with providing an explicit schema
    spark.read.format("redis")
      .schema(
        StructType(Array(
          StructField("id", IntegerType),
          StructField("name", StringType),
          StructField("age", IntegerType))
        )
      )
      .option("keys.pattern", "customer:*")
      .option("key.column", "id")
      .load().show()

    // read data with automatically infer the schema
    spark.read.format("redis")
      .option("keys.pattern", "customer:*")
      .option("infer.schema", value = true)
      .load().show()

    // *********** write the redis hash type data with DataFrame *******************
    val personSeq = Seq(Person(1, "John", 30), Person(2, "Peter", 45))
    val df = spark.createDataFrame(personSeq)
    df.write.format("redis")
      .option("table", "person")
      .option("key.column", "id")
      .mode(SaveMode.Overwrite)
      .save()

    // Read data via redis cli:
    // redis:6379> HGETALL person:1
    // age
    // 30
    // name
    // John

    spark.read.format("redis")
      .option("table", "person")
      .option("key.column", "id")
      .load().show(truncate = false)

    spark.stop()
  }
}
